// DiscoverServer.cpp: implementation of the CDiscoverServer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SDRxxServer.h"
#include "DiscoverServer.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/*------------------------------------------------------------------*/
/*----------------> L O C A L   D E F I N E S <---------------------*/
/*------------------------------------------------------------------*/
#define KEY0 0x5A
#define KEY1 0xA5
#define MSG_REQ 0
#define MSG_RESP 1
#define MSG_SET 2



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDiscoverServer::CDiscoverServer()
{
	m_hRxSocket = INVALID_SOCKET;
	m_hTxSocket = INVALID_SOCKET;
	m_UDPThreadRunning = FALSE;
	m_pUDPThread = NULL;
	m_IPAdr = 0;
	m_Port = 0;
	m_SdrSN = "";
	m_SdrName = "";
	m_BootVer = 0;
	m_FirmVer = 0;
	m_InUse = FALSE;
}

CDiscoverServer::~CDiscoverServer()
{
	Stop();
}


//////////////////////////////////////////////////////////////////////
// Worker Thread static procedure that just calls the main Socket process loop
//////////////////////////////////////////////////////////////////////
UINT CDiscoverServer::UDPThreadProc(LPVOID pParam)
{
CDiscoverServer* pObject = (CDiscoverServer*)pParam;
	pObject->UDPThread();
	return 11;
}


/////////////////////////////////////////////////////////////////////////////
//Start up Bcst UDP Rx Socket Worker thread
/////////////////////////////////////////////////////////////////////////////
void CDiscoverServer::Start()
{
	m_pUDPThread = AfxBeginThread(UDPThreadProc,
								this,
								THREAD_PRIORITY_HIGHEST,
								0,
								CREATE_SUSPENDED);
	m_pUDPThread->m_bAutoDelete = FALSE;
	m_UDPThreadRunning = TRUE;
	m_pUDPThread->ResumeThread();
}

//////////////////////////////////////////////////////////////////////
//Stops UDP Rx Socket Worker thread
//////////////////////////////////////////////////////////////////////
void CDiscoverServer::Stop() 
{
	m_UDPThreadRunning = FALSE;
	if(m_pUDPThread)
	{
		m_pUDPThread->ResumeThread();
		::WaitForSingleObject(m_pUDPThread->m_hThread, INFINITE);
		delete m_pUDPThread;
		m_pUDPThread = NULL;
	}
}

//////////////////////////////////////////////////////////////////////
//  Main processing loop of this IO object(run in worker thread)
// This is a worker thread that services the TCP/IP Socket.
//////////////////////////////////////////////////////////////////////
void CDiscoverServer::UDPThread()
{
struct sockaddr_in saudp;
fd_set ReadFDs, ExceptFDs;
char RxBuf[2048];

int nRet;
struct timeval timeout;
	timeout.tv_sec =  1;		// select will wait for 1 second at a time
	timeout.tv_usec = 0;

//Create UDP sockets
	//create Rx socket
	if( (m_hRxSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP) ) == INVALID_SOCKET)
	{
		TRACE("\r\nSocket failed to create\n\r");
		return;
	}
	BOOL optval=TRUE;
	setsockopt( m_hRxSocket, SOL_SOCKET, SO_REUSEADDR, (const char*)&optval, sizeof(optval) );
	setsockopt( m_hRxSocket, SOL_SOCKET, SO_DONTLINGER, (const char*)&optval, sizeof(optval) );
	setsockopt( m_hRxSocket, SOL_SOCKET, SO_BROADCAST,(const char*) &optval, sizeof(optval) );

	memset( &saudp, 0, sizeof(saudp));
	saudp.sin_family = AF_INET;
	saudp.sin_addr.S_un.S_addr = htonl(INADDR_ANY );
	saudp.sin_port = htons(DISCOVER_SERVER_PORT);

	nRet = bind(m_hRxSocket, (sockaddr *)&saudp, sizeof(saudp));
	if(nRet==SOCKET_ERROR)
	{
		TRACE("\r\nSocket failed to bind\n\r");
		return;
	}
	ULONG val = 1;
	ioctlsocket(m_hRxSocket, FIONBIO, &val);	//set non-blocking mode


	//create Tx socket
	if( (m_hTxSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP) ) == INVALID_SOCKET)
	{
		TRACE("\r\nSocket failed to create\n\r");
	}

	optval=TRUE;
	setsockopt( m_hTxSocket, SOL_SOCKET, SO_DONTLINGER, (const char*)&optval, sizeof(optval) );
	setsockopt( m_hTxSocket, SOL_SOCKET, SO_BROADCAST,(const char*) &optval, sizeof(optval) );

	//main thread loop to service UDP broadcast Reads
	while(m_UDPThreadRunning)
	{
		SetupFDSets(ReadFDs, ExceptFDs);
		int ret = select(0, &ReadFDs, NULL, &ExceptFDs, &timeout);
        if( ret > 0)
		{
			// Something happened on the socket.
			// See if this  socket's flag is set in any of the FD sets.
			if (FD_ISSET(m_hRxSocket, &ExceptFDs))
			{
				FD_CLR(m_hRxSocket, &ExceptFDs);
				TRACE(" Exception\n");
				m_UDPThreadRunning = FALSE;
			}
			else
			{
				if (FD_ISSET(m_hRxSocket, &ReadFDs))
				{	//is a client read event
					FD_CLR(m_hRxSocket, &ReadFDs);
					int nBytes = recv(m_hRxSocket, RxBuf,2048 , 0);
					if(nBytes<=0)
					{	//read event with no bytes means client closed socket
						TRACE(" Error\n");
						m_UDPThreadRunning = FALSE;
					}
					ParseDiscover(RxBuf, nBytes);
				}
			}
		}
		else
		{	//here with a select error or just a timeout
			if(ret == SOCKET_ERROR)
			{
				TRACE("select error\n");
				m_UDPThreadRunning = FALSE;
				return;
			}
		}
	}
	CloseSockets();
//	TRACE("thread exit\n");

}


//////////////////////////////////////////////////////////////////////
// clears and sets the select events to wait for
//////////////////////////////////////////////////////////////////////
void CDiscoverServer::SetupFDSets(fd_set& ReadFDs,  fd_set& ExceptFDs)
{
	FD_ZERO(&ReadFDs);
	FD_ZERO(&ExceptFDs);

	// Add client connection events as needed
	if(	m_hRxSocket != INVALID_SOCKET )
	{
		FD_SET(m_hRxSocket, &ExceptFDs);
		FD_SET(m_hRxSocket, &ReadFDs);
	}
}



/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                  ---------------------------                              */
/*                 | P a r s e D i s c o v e r |                             */
/*                  ---------------------------                              */
/*    Called from UDP data receive thread to parse a received  msg.          */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void CDiscoverServer::ParseDiscover( char* Buf, int Len )
{
	if(Len>sizeof(m_Discovermsg))	//if msg too long abort
	{
		TRACE("rx msg too long\n");
		return;
	}

	//copy UDP packet into m_Discovermsg structure
	memset((void*)&m_Discovermsg,0,sizeof(m_Discovermsg));
	memcpy((void*)&m_Discovermsg, (void*)Buf, Len );
	//if .name field is filled in, see if name matches
	if(m_Discovermsg.name[0] )
	{
		m_Str = m_Discovermsg.name;
		if(m_Str != m_SdrName)
			return;			//not us
	}
	//if .sn field is filled in, see if sn matches
	if(m_Discovermsg.sn[0] )
	{
		m_Str = m_Discovermsg.sn;
		if(m_Str != m_SdrSN)
			return;		//not us
	}
	if( (m_Discovermsg.key[0]!=KEY0) || (m_Discovermsg.key[1]!=KEY1) )
	{	//if not a valid Discover packet
		TRACE("rx msg not valid\n");
		return;
	}
	//here with valid msg
	if( MSG_REQ==m_Discovermsg.op)
		SendSettings();
	else if(MSG_SET==m_Discovermsg.op)
		SetSettings();
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                  -------------------------                                */
/*                 | S e n d S e t t i n g s |                               */
/*                  -------------------------                                */
/* Send current settings message back to a client.                           */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void CDiscoverServer::SendSettings(void)
{
	
	strcpy( m_Discovermsg.name, m_SdrName);	//fill in name string field
	strcpy( m_Discovermsg.sn, m_SdrSN);	//fill in Serial Number string field
		

	m_Discovermsg.length[0] = sizeof(m_Discovermsg)&0xFF;
	m_Discovermsg.length[1] = sizeof(m_Discovermsg)>>8;
	m_Discovermsg.key[0] = KEY0;
	m_Discovermsg.key[1] = KEY1;
	m_Discovermsg.op = MSG_RESP;

	//fill in IP adr and port
	m_Discovermsg.ipaddr[0] = m_IPAdr & 0x000000FF;
	m_Discovermsg.ipaddr[1] = (m_IPAdr>>8) & 0x000000FF;
	m_Discovermsg.ipaddr[2] = (m_IPAdr>>16) & 0x000000FF;
	m_Discovermsg.ipaddr[3] = (m_IPAdr>>24) & 0x000000FF;

	m_Discovermsg.port[0] = m_Port&0x00FF;
	m_Discovermsg.port[1] = (m_Port>>8)&0x00FF;


	m_Discovermsg.btver[0] = m_BootVer&0x00FF;
	m_Discovermsg.btver[1] = (m_BootVer>>8)&0x00FF;

	m_Discovermsg.fwver[0] = m_FirmVer&0x00FF;
	m_Discovermsg.fwver[1] = (m_FirmVer>>8)&0x00FF;

	m_Discovermsg.connection[0] = 'U';
	m_Discovermsg.connection[1] = 'S';
	m_Discovermsg.connection[2] = 'B';

	if(m_InUse)
		m_Discovermsg.status |= STATUS_BIT_CONNECTED;
	else
		m_Discovermsg.status &= ~STATUS_BIT_CONNECTED;

	SendDiscoverData( &m_Discovermsg);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                  -----------------------                                  */
/*                 | S e t S e t t i n g s |                                 */
/*                  -----------------------                                  */
/* Set Settings to values in Discover msg from client.                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void CDiscoverServer::SetSettings(void)
{
#if 0
char Name[16];
char SN[16];
tIPadr ipadr;
U8BIT ch;
int i;
	//Make sure this set message is for us
	//Get our name and serial number
	i = 0;
	do{
		ch = ID_NAME[i];
		Name[i++] = ch;
	}while( (ch != 0) &&  (i<16) );
	i = 0;
	do{
		ch = SERIAL_NUMBER[i];
		SN[i++] = ch;
	}while( (ch != 0) &&  (i<16) );

	if( strcmp(Discovermsg.name, Name) || strcmp(Discovermsg.sn, SN) )
		return;		//msg not for us


	Settings.TcpPort = Discovermsg.portadr[1];
	Settings.TcpPort <<= 8;
	Settings.TcpPort += Discovermsg.portadr[0];
#endif
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                --------------------------------                           */
/*               | S e n d D i s c o v e r D a t a |                         */
/*                ---------------------------------                          */
/* Send discover msg to client.                                              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void CDiscoverServer::SendDiscoverData(tDiscover_MSG* msg )
{
struct sockaddr_in saudp;
	if( INVALID_SOCKET != m_hTxSocket)
	{
		int length = msg->length[1];
		length <<= 8;
		length += (int)msg->length[0];
		memset( &saudp, 0, sizeof(saudp)-1 );
		saudp.sin_family = AF_INET;
		saudp.sin_addr.S_un.S_addr = inet_addr("255.255.255.255");
		saudp.sin_port = htons(DISCOVER_CLIENT_PORT);

		int bytessent = sendto(m_hTxSocket, (const char*)msg, length, 0, (SOCKADDR *)&saudp, sizeof(saudp) );
		if( bytessent != length)
			TRACE("Send Failed\n");
		else
			TRACE("Send Msg\n");
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                             --------------                                */
/*                            | CloseSockets |                               */
/*                             --------------                                */
/* Closes the sockets                                                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void CDiscoverServer::CloseSockets()
{
int nNewBytes;
char acReadBuffer[256];
	if(m_hRxSocket != INVALID_SOCKET)
	{
		if(shutdown(m_hRxSocket, SD_BOTH ) == SOCKET_ERROR)
		{
			closesocket(m_hRxSocket);
		}
		else
		{
			nNewBytes = recv(m_hRxSocket, acReadBuffer, 256, 0);
			closesocket(m_hRxSocket);
		}
	}
	m_hRxSocket = INVALID_SOCKET;
	//
	if(m_hTxSocket != INVALID_SOCKET)
	{
		if(shutdown(m_hTxSocket, SD_BOTH ) == SOCKET_ERROR)
		{
			closesocket(m_hTxSocket);
		}
		else
		{
			nNewBytes = recv(m_hTxSocket, acReadBuffer, 256, 0);
			closesocket(m_hTxSocket);
		}
	}
	m_hTxSocket = INVALID_SOCKET;

}